==> gambas2-1.9.15.tar.bz2: gambas2-1.9.15/gb.qt/src/ext/gdocument.h <==
/***************************************************************************

  gdocument.h

  (c) 2000-2005 Benoît Minisini <gambas@users.sourceforge.net>

  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 1, or (at your option)
  any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

***************************************************************************/

#ifndef __GDOCUMENT_H
#define __GDOCUMENT_H

#include <qptrlist.h>
#include <qstring.h>
#include <qgridview.h>
#include <qtimer.h>

struct GHighlightStyle
{
  QColor color;
  bool bold;
  bool italic;
};

struct GHighlight
{
  unsigned state : 4;
  unsigned len : 12;
};

typedef
  QMemArray<GHighlight> GHighlightArray;

class GLine
{
public:

  enum
  {
    Background = 0, Normal, Keyword, Subr,
    Operator, Symbol, Number, String,
    Comment, Breakpoint, Current, Datatype,
    Selection, Highlight, Line, Error,
    NUM_STATE
  };

  enum
  {
    BreakpointFlag = 1,
    CurrentFlag = 2
  };

  QString s;
  GHighlightArray highlight;
  unsigned state : 4;
  unsigned modified : 1;
  unsigned changed : 1;
  unsigned flag : 2;
  unsigned proc : 1;
  unsigned baptized : 1;
  unsigned _reserved : 22;

  GLine() { s = ""; state = 0; flag = 0; modified = false; changed = false; baptized = false; proc = false;  }
};

typedef
  int (*GHighlightCallback)(int state, char *line, long length, GHighlight **highlight);

class GCommand;
class GEditor;


class GDocument : public QObject
{
  Q_OBJECT

private:

  static bool isWordChar(QChar c) { return c.isLetterOrNumber() || c == '_' || c == '$'; }
  static int convState(int state);

  QPtrList<GCommand> undoList;
  QPtrList<GCommand> redoList;
  int highlightMode;
  GHighlightCallback highlightCallback;
  int maxLength;
  int oldMaxLength;
  uint oldCount;
  bool readOnly;
  bool blockUndo;
  GEditor *selector;
  int xs, ys, xs2, ys2;
  int tabWidth;

  void clearUndo();
  void addUndo(GCommand *);
  void addRedo(GCommand *);
  void updateViews(int row = -1, int count = 1);

public:

  enum GHighlightMode
  {
    None = 0,
    Gambas = 1,
    Custom = 2
  };

  static bool analyze(QString &s, GHighlightArray &data, bool *proc);

  QPtrList<GLine> *lines;
  QPtrList<GEditor> *views;
  int xAfter, yAfter;

  GDocument();
  ~GDocument();

  void clear();
  void reset();
  QString getText();
  void setText(const QString & text);
  int getLength() const;

  void subscribe(GEditor *view);
  void unsubscribe(GEditor *view);

  void setReadOnly(bool ro) { readOnly = ro; };
  bool isReadOnly() const { return readOnly; }

  void setTabWidth(int tw) { tabWidth = tw; }
  int getTabWidth() const { return tabWidth; }

  int getMaxLineLength() const { return maxLength; }

  QString getLine(int y) const;
  void setLine(int y, QString & str);

  bool getLineFlag(int y, int f) const;
  void setLineFlag(int y, int f, bool b);

  int lineLength(int y) { return lines->at(y)->s.length(); }
  int numLines() { return lines->count(); }

  void insert(int y, int x, const QString & str);
  void remove(int y, int x, int y2, int x2);

  bool undo();
  bool redo();
  void begin();
  void end();

  void setHighlighMode(int mode) { highlightMode = mode; updateViews(); }
  void setHighlightCallback(GHighlightCallback callback) { highlightCallback = callback; }
  int getHighlightMode() const { return highlightMode; }
  bool colorize(int y);

  int getIndent(int y, bool *empty = NULL);
  int wordLeft(int y, int x, bool word);
  int wordRight(int y, int x, bool word);

  QString getSelectedText() const;

  bool hasSelection() const { return selector != NULL; }
  bool hasSelection(GEditor *view) const { return selector == view; }
  void getSelection(int *y1, int *x1, int *y2, int *x2) const;
  void startSelection(GEditor *view, int y, int x);
  void endSelection(int y, int x);
  void hideSelection();
  void eraseSelection();

  void copy() const;

signals:

  void textChanged();
};


class GEditor : public QGridView
{
  Q_OBJECT

  friend class GDocument;

private:

  static QPixmap *cache;
  static int count;
  static const QEvent::Type EVENT_ENSURE_VISIBLE = QEvent::User;

  GDocument *doc;
  int charWidth;
  int x, y, xx;
  bool cursor;
  QTimer *blinkTimer;
  QTimer *scrollTimer;
  QPoint lastPos;
  int x1m, x2m, ym;
  int margin;
  int lineNumberLength;

  GHighlightStyle styles[GLine::NUM_STATE];
  int flags;

  int lineLength(int y) { return doc->lines->at(y)->s.length(); }
  int numLines() { return doc->lines->count(); }
  void startBlink();
  void stopBlink();
  void updateLength();
  void updateLine(int y) { updateCell(y, 0); }
  void updateMargin();

  void paintText(QPainter &p, GLine *l, int x, int y);

private slots:

  void blinkTimerTimeout();
  void scrollTimerTimeout();
	void docTextChanged();

protected:

  virtual void paintCell(QPainter *, int row, int);
  virtual void fontChange(const QFont &oldFont);
  virtual void keyPressEvent(QKeyEvent *e);
  virtual void mousePressEvent(QMouseEvent *e);
  virtual void mouseReleaseEvent(QMouseEvent *e);
  virtual void mouseMoveEvent(QMouseEvent *e);
  virtual void mouseDoubleClickEvent(QMouseEvent *e);
  virtual void resizeEvent(QResizeEvent *e);
  virtual void focusInEvent(QFocusEvent *);
  virtual void focusOutEvent(QFocusEvent *);
  virtual bool event(QEvent *e);

public:

  enum Flag
  {
    ShowProcedureLimits = 1,
    DrawWithRelief = 2,
    ShowModifiedLines = 3,
    ShowCurrentLine = 4,
    ShowLineNumbers = 5,
    HighlightBraces = 6
  };

  GEditor(QWidget *parent);
  ~GEditor();

  void setDocument(GDocument *doc);
  GDocument *getDocument() const { return doc; }

  void getCursor(int *yc, int *xc) const { *yc = y; *xc = x; }
  void insert(QString text);
  bool cursorGoto(int ny, int nx, bool mark);
  void cursorLeft(bool shift, bool ctrl);
  void cursorRight(bool shift, bool ctrl);
  void cursorUp(bool mark);
  void cursorDown(bool mark);
  void cursorPageUp(bool mark);
  void cursorPageDown(bool mark);
  void cursorHome(bool shift, bool ctrl);
  void cursorEnd(bool shift, bool ctrl);
  void newLine();
  void backspace();
  void del();
  void copy();
  void cut();
  void paste();
  void undo();
  void redo();
  void tab(bool back);
  void selectAll();

  void setStyle(int index, GHighlightStyle *style);
  void getStyle(int index, GHighlightStyle *style);
  int getFlag(int f) const { return flags & (1 << f); }
  void setFlag(int f, bool v);

  int getLineHeight() const { return cellHeight(); }
  int getCharWidth() const { return charWidth; }
  void cursorToPos(int y, int x, int *px, int *py);
  void posToCursor(int px, int py, int *y, int *x);
  void checkMatching();

signals:

  void cursorMoved();
  void textChanged();
};

#endif

==> gambas2-1.9.20.tar.bz2: gambas2-1.9.20/gb.qt/src/ext/gdocument.h <==
/***************************************************************************

  gdocument.h

  (c) 2000-2005 Benoît Minisini <gambas@users.sourceforge.net>

  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 1, or (at your option)
  any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

***************************************************************************/

#ifndef __GDOCUMENT_H
#define __GDOCUMENT_H

#include <qptrlist.h>
#include <qstring.h>
#include <qgridview.h>
#include <qtimer.h>

struct GHighlightStyle
{
  QColor color;
  bool bold;
  bool italic;
  bool underline;
};

struct GHighlight
{
  unsigned state : 4;
  unsigned len : 12;
};

typedef
  QMemArray<GHighlight> GHighlightArray;

class GLine
{
public:

  enum
  {
    Background = 0, Normal, Keyword, Subr,
    Operator, Symbol, Number, String,
    Comment, Breakpoint, Current, Datatype,
    Selection, Highlight, Line, Error,
    NUM_STATE
  };

  enum
  {
    BreakpointFlag = 1,
    CurrentFlag = 2
  };

  QString s;
  GHighlightArray highlight;
  unsigned state : 4;
  unsigned modified : 1;
  unsigned changed : 1;
  unsigned flag : 2;
  unsigned proc : 1;
  unsigned baptized : 1;
  unsigned _reserved : 22;

  GLine() { s = ""; state = Normal; flag = 0; modified = false; changed = false; baptized = false; proc = false;  }
};

class GCommand;
class GEditor;

typedef
  bool (*GHighlightCallback)(GEditor *master, uint &state, QString &s, GHighlightArray &highlight, bool &proc);


class GDocument : public QObject
{
  Q_OBJECT

private:

  static bool isWordChar(QChar c) { return c.isLetterOrNumber() || c == '_' || c == '$'; }
  static int convState(int state);

  QPtrList<GCommand> undoList;
  QPtrList<GCommand> redoList;
  int highlightMode;
  GHighlightCallback highlightCallback;
  int maxLength;
  int oldMaxLength;
  uint oldCount;
  bool readOnly;
  bool blockUndo;
  GEditor *selector;
  int xs, ys, xs2, ys2;
  int tabWidth;
  int baptismLimit;

  void clearUndo();
  void addUndo(GCommand *);
  void addRedo(GCommand *);
  void updateViews(int row = -1, int count = 1);

public:

  enum GHighlightMode
  {
    None = 0,
    Gambas = 1,
    Custom = 2
  };

  static bool highlightGambas(GEditor *master, uint &state, QString &s, GHighlightArray &data, bool &proc);
  static bool highlightTest(GEditor *master, uint &state, QString &s, GHighlightArray &data, bool &proc);

  QPtrList<GLine> *lines;
  QPtrList<GEditor> *views;
  int xAfter, yAfter;

  GDocument();
  ~GDocument();

  void clear();
  void reset();
  QString getText();
  void setText(const QString & text);
  int getLength() const;

  void subscribe(GEditor *view);
  void unsubscribe(GEditor *view);

  void setReadOnly(bool ro) { readOnly = ro; };
  bool isReadOnly() const { return readOnly; }

  void setTabWidth(int tw) { tabWidth = tw; }
  int getTabWidth() const { return tabWidth; }

  int getMaxLineLength() const { return maxLength; }

  QString getLine(int y) const;
  void setLine(int y, QString & str);

  bool getLineFlag(int y, int f) const;
  void setLineFlag(int y, int f, bool b);

  int lineLength(int y) { return lines->at(y)->s.length(); }
  int numLines() { return lines->count(); }

  void insert(int y, int x, const QString & str);
  void remove(int y, int x, int y2, int x2);

  bool undo();
  bool redo();
  void begin();
  void end();

  void setHighlightMode(int mode, GHighlightCallback cb = 0);
  int getHighlightMode() const { return highlightMode; }
  void colorize(int y);
  void baptizeUntil(int y);

  int getIndent(int y, bool *empty = NULL);
  int wordLeft(int y, int x, bool word);
  int wordRight(int y, int x, bool word);

  QString getSelectedText() const;

  bool hasSelection() const { return selector != NULL; }
  bool hasSelection(GEditor *view) const { return selector == view; }
  void getSelection(int *y1, int *x1, int *y2, int *x2) const;
  void startSelection(GEditor *view, int y, int x);
  void endSelection(int y, int x);
  void hideSelection();
  void eraseSelection();

  void copy() const;

signals:

  void textChanged();
};


class GEditor : public QGridView
{
  Q_OBJECT

  friend class GDocument;

private:

  static QPixmap *cache;
  static int count;
  static const QEvent::Type EVENT_ENSURE_VISIBLE = QEvent::User;

  GDocument *doc;
  int charWidth;
  int x, y, xx;
  bool cursor;
  QTimer *blinkTimer;
  QTimer *scrollTimer;
  int x1m, x2m, ym;
  int margin;
  int lineNumberLength;

	QFont italicFont;
  GHighlightStyle styles[GLine::NUM_STATE];
  int flags;

  int lineLength(int y) { return doc->lines->at(y)->s.length(); }
  int numLines() { return doc->lines->count(); }
  void startBlink();
  void stopBlink();
  void updateLength();
  void updateLine(int y) { updateCell(y, 0); }
  void updateMargin();

  void paintText(QPainter &p, GLine *l, int x, int y, int xmin, int lmax);

private slots:

  void blinkTimerTimeout();
  void scrollTimerTimeout();
	void docTextChanged();
	void baptizeVisible();

protected:

  virtual void paintCell(QPainter *, int row, int);
  virtual void fontChange(const QFont &oldFont);
  virtual void keyPressEvent(QKeyEvent *e);
  virtual void mousePressEvent(QMouseEvent *e);
  virtual void mouseReleaseEvent(QMouseEvent *e);
  virtual void mouseMoveEvent(QMouseEvent *e);
  virtual void mouseDoubleClickEvent(QMouseEvent *e);
  virtual void resizeEvent(QResizeEvent *e);
  virtual void focusInEvent(QFocusEvent *);
  virtual void focusOutEvent(QFocusEvent *);
  virtual bool event(QEvent *e);

public:

  enum Flag
  {
    ShowProcedureLimits = 1,
    DrawWithRelief = 2,
    ShowModifiedLines = 3,
    ShowCurrentLine = 4,
    ShowLineNumbers = 5,
    HighlightBraces = 6,
    HighlightCurrent = 7
  };

  GEditor(QWidget *parent);
  ~GEditor();

  void setDocument(GDocument *doc);
  GDocument *getDocument() const { return doc; }

  void getCursor(int *yc, int *xc) const { *yc = y; *xc = x; }
  void insert(QString text);
  bool cursorGoto(int ny, int nx, bool mark);
  void cursorLeft(bool shift, bool ctrl);
  void cursorRight(bool shift, bool ctrl);
  void cursorUp(bool shift, bool ctrl);
  void cursorDown(bool shift, bool ctrl);
  void cursorPageUp(bool mark);
  void cursorPageDown(bool mark);
  void cursorHome(bool shift, bool ctrl);
  void cursorEnd(bool shift, bool ctrl);
  void newLine();
  void backspace();
  void del();
  void copy();
  void cut();
  void paste();
  void undo();
  void redo();
  void tab(bool back);
  void selectAll();

  void setStyle(int index, GHighlightStyle *style);
  void getStyle(int index, GHighlightStyle *style);
  int getFlag(int f) const { return flags & (1 << f); }
  void setFlag(int f, bool v);

  int getLineHeight() const { return cellHeight(); }
  int getCharWidth() const { return charWidth; }
  void cursorToPos(int y, int x, int *px, int *py);
  void posToCursor(int px, int py, int *y, int *x);
	int lastVisibleRow() { return rowAt(contentsY() + visibleHeight() - 1); }

  void checkMatching();

signals:

  void cursorMoved();
  void textChanged();
};

#endif
